/*******************************************************************************
   RoBoard COM Port Simple Tool

     This program is for the purpose of enabling/disabling the boosting mode 
     of RoBoard's native COM port. 
     Execute it without arguments to see how to use.

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined(linux)
    #define stricmp  strcasecmp
#endif

#ifdef WINCE
    #include <windows.h>
	#define stricmp _stricmp
#endif

#if defined(WINCE) || defined(WIN32)
    #include <roboard_dll.h>  // use the DLL version of RoBoIO lib
    // #include <roboard.h>   // use the static version of RoBoIO lib
#else
    #include <roboard.h>
#endif


void help(char* s) {
    int l = strlen(s);

    for (; l>=0; l--) if ((s[l] == '/') || (s[l] == '\\')) break;
    if (l >= 0) s = &s[l+1];

	printf(
	    "Usage                                                  \n"
	    "=======                                                \n"
	    "%s [ver] [com] [switch]                                \n"
	    "                                                       \n"
	    "   [ver] (RoBoard version)                             \n"
	    "      RB050   : RoBoard RB-050                         \n"
	    "      RB110   : RoBoard RB-110                         \n"
	    "      RB100RD : RoBoard RB-100RD                       \n"
	    "      (note: only RB-100RD/RB-110/RB-050 is supported) \n"
	    "                                                       \n"
	    "   [com]:                                              \n"
	    "      COM1 : RoBoard's COM1                            \n"
	    "      COM2 : RoBoard's COM2                            \n"
	    "      COM3 : RoBoard's COM3                            \n"
	    "      COM4 : RoBoard's COM4                            \n"
	    "                                                       \n"
	    "   [switch]:                                           \n"
	    "      0 : disable boosting mode                        \n"
	    "      1 : enable boosting mode                         \n"
	    "                                                       \n"
	    "(Press Enter to continue...)                           \n"
	    , s
    ); 
    getchar();

	printf(
	    "example                                                        \n"
	    "=========                                                      \n"
	    "%s RB110 COM2 0                                                \n"
	    "    disable the boosting mode of RB-110's COM2.                \n"
	    "                                                               \n"
	    "%s RB050 COM3 1                                                \n"
	    "    enable the boosting mode of RB-050's COM3.                 \n"
	    "                                                               \n"
	    , s, s
    );
}


int GetCOMNum(char* str) {
	if(stricmp("COM1", str) == 0)
		return 0;
	else
	if(stricmp("COM2", str) == 0)
		return 1;
	else
	if(stricmp("COM3", str) == 0)
		return 2;
	else
	if(stricmp("COM4", str) == 0)
		return 3;

	printf("ERR: wrong COM port parameter!!!\n\n");
    exit(1);
}

int GetSwitch(char* str) {
	if(strcmp("0", str) == 0)
		return 0;
	else
	if(strcmp("1", str) == 0)
		return 1;

    printf("ERR: wrong switch parameter!!!\n\n");
    exit(1);
}

void SetRBVer(int argc, char* argv[]) {
    if (stricmp(argv[1], "RB050") == 0)
    {
        roboio_SetRBVer(RB_050); return;
    }
    else
    if (stricmp(argv[1], "RB110") == 0)
    {
        roboio_SetRBVer(RB_110); return;
    }
    else
    if (stricmp(argv[1], "RB100RD") == 0)
    {
        roboio_SetRBVer(RB_100RD); return;
    }

	printf("ERR: unsupported RoBoard version!!!\n\n");
    exit(1);
}

#ifdef WINCE
int _tmain(int argc, _TCHAR* _argv[])
{
	char  argvbuf[50][100] = {'\0'};
	char* argv[50];

	if (argc > 50) argc = 50;
	for (int _argc=0; _argc<argc; _argc++)
	{
		argv[_argc] = &(argvbuf[_argc][0]);
		wcstombs(argv[_argc], _argv[_argc], 100);
	}
#else
int main(int argc, char* argv[])
{
#endif

	int comnum, mode;

	if(argc == 1)
	{
		help(argv[0]);
		return 0;
	}

	if(argc != 4)
	{
		printf("ERR: wrong number of parameters!!!\n\n");
		return 1;
	}

	SetRBVer(argc, argv);

	comnum = GetCOMNum(argv[2]);
	mode   = GetSwitch(argv[3]);


    if (io_init() == false)
	{
        printf("ERR: fail to initialize I/O lib (%s)!\n", roboio_GetErrMsg());
		return 1;
	}

    if (mode == 0)
    {
        printf("Disable the boosting mode of COM%d\n", comnum+1);
        switch (comnum)
        {
            case 0:
                com1_DisableTurboMode(); break;
            case 1:
                com2_DisableTurboMode(); break;
            case 2:
                com3_DisableTurboMode(); break;
            case 3:
                com4_DisableTurboMode(); break;
        }
    }
    else
    {
        printf("Enable the boosting mode of COM%d\n", comnum+1);
        switch (comnum)
        {
            case 0:
                com1_EnableTurboMode(); break;
            case 1:
                com2_EnableTurboMode(); break;
            case 2:
                com3_EnableTurboMode(); break;
            case 3:
                com4_EnableTurboMode(); break;
        }
    }

    io_close();
	return 0;
}

